use std::path::PathBuf;

pub mod ambient;
pub mod config;

#[derive(Debug, thiserror::Error)]
pub enum AdapterError {
    #[error("failed to write message reporting run has been triggered")]
    Triggered(#[source] radicle_ci_broker::msg::helper::MessageHelperError),

    #[error("failed to write response message reporting success")]
    Succeded(#[source] radicle_ci_broker::msg::helper::MessageHelperError),

    #[error("failed to write response message reporting failure")]
    Failed(#[source] radicle_ci_broker::msg::helper::MessageHelperError),

    #[error("failed to read request from stdin")]
    ReadRequest(#[source] radicle_ci_broker::msg::helper::MessageHelperError),

    #[error(transparent)]
    Config(#[from] crate::config::ConfigError),

    #[error("failed to extract commit from request message")]
    Commit(#[source] radicle_ci_broker::msg::MessageError),

    #[error("failed to create a temporary directory")]
    TempDir(#[source] std::io::Error),

    #[error("failed to load Radicle profile")]
    Profile(#[source] radicle::profile::Error),

    #[error("failed to invoke command {0}")]
    Invoke(&'static str, #[source] std::io::Error),

    #[error("failed to serialize Ambient project list to YAML")]
    ToYaml(#[source] serde_yml::Error),

    #[error("failed to write Ambient project list to file")]
    WriteProjects(#[source] std::io::Error),

    #[error("failed to create directory {0}")]
    CreateDirs(PathBuf, #[source] std::io::Error),

    #[error("failed to write run log to file")]
    WriteRunLog(PathBuf, #[source] std::io::Error),

    #[error("failed to read CI plan from project {0}")]
    ReadPlan(PathBuf, #[source] std::io::Error),

    #[error("failed to parse CI plan as YAML fro file {0}")]
    ParsePlan(PathBuf, #[source] serde_yml::Error),

    #[error("failed to serialize trigger message as JSON")]
    ToJson(#[source] serde_json::Error),

    #[error("failed to write temporary configuration file")]
    WriteConfig(#[source] std::io::Error),

    #[error("failed to run ambient-driver to get its configurarion")]
    GetAmbientConfig,

    #[error(transparent)]
    AdminLog(#[from] radicle_ci_broker::msg::helper::LogError),

    #[error(transparent)]
    MessageHelper(#[from] radicle_ci_broker::msg::helper::MessageHelperError),
}
